<?php
/**
 * @package     DOCman
 * @copyright   Copyright (C) 2011 - 2015 Timble CVBA. (http://www.timble.net)
 * @license     GNU GPLv3 <http://www.gnu.org/licenses/gpl.html>
 * @link        http://www.joomlatools.com
 */

/**
 * License Controller Behavior
 *
 * @author  Ercan Ozkaya <https://github.com/ercanozkaya>
 * @package Joomlatools\Plugin\DOCman
 */
class PlgDocmanLicenseControllerBehaviorLicensable extends KControllerBehaviorAbstract
{
	protected function _initialize(KObjectConfig $config)
	{
		$config->append(array(
			'priority'   => KCommandHandlerInterface::PRIORITY_HIGH,
			'article_id' => 0,
			'title'      => '',
			'text'       => '',
			'pages'      => array(),
			'categories' => array()
		));

		parent::_initialize($config);
	}

	/**
	 * Display license on the selected pages.
	 *
	 * @param KControllerContextInterface $context
	 * @return bool
	 */
	protected function _beforeRender(KControllerContextInterface $context)
	{
		$config     = $this->getConfig();
		$pages      = KObjectConfig::unbox($config->pages);
		$categories = KObjectConfig::unbox($config->categories);

		if (!$pages || in_array($this->getRequest()->query->get('Itemid', 'int'), (array)$pages))
		{
			if (!$categories || in_array($this->getModel()->fetch()->docman_category_id, (array)$categories))
			{
				$url   = clone $this->getRequest()->getUrl();
				$query = $url->getQuery(true);

				if (empty($query['accept_license']) && $this->_getLicense()->text)
				{
					$template = 'file://plugins/docman/license/view/default';
					$override = 'com://site/docman.license.default';

					if ($this->getObject('template.locator.factory')->createLocator($override)->locate($override . '.html'))
					{
						$template = $override;
					}

					$url->query['accept_license'] = 1;

					$html = $this->getObject('com:koowa.view.html', array(
						'layout' => $template
					))->render(array(
						'license'      => $this->_getLicense(),
						'download_url' => $url
					));

					$this->getResponse()->setContent($html);

					return false;
				}
			}
		}

		return true;
	}

	/**
	 * Returns license title and text
	 *
	 * @return stdClass
	 */
	protected function _getLicense()
	{
		$config = $this->getConfig();

		if ($config->article_id)
		{
			try
			{
				$article = $this->getObject('com://admin/articles.database.table.content', array(
					'name'            => 'content',
					'identity_column' => 'id'
				))->select(array('id' => $config->article_id), KDatabase::FETCH_ROW);

				if ($article && $article->id)
				{
					$config->title = $article->title;
					$config->text  = $article->introtext . $article->fulltext;
				}
			} catch (Exception $e)
			{
			}
		}

		$license        = new stdClass;
		$license->title = $config->title;
		$license->text  = JHtml::_('content.prepare', $config->text);

		return $license;
	}

}