<?php
/**
 * @package     Docman.Plugin
 * @subpackage  Leads
 * @copyright   Copyright (C) 2011 - 2016 Timble CVBA. (http://www.timble.net)
 * @license     GNU GPLv3 <http://www.gnu.org/licenses/gpl.html>
 * @link        http://www.joomlatools.com
 */

/**
 * Captcha Controller Behavior
 *
 * @package Joomlatools\Plugin\DOCman
 */
class PlgDocmanLeadsControllerBehaviorCaptchable extends KControllerBehaviorAbstract
{
    /**
     * Secret key for the site.
     * @var string
     */
    private $secret_key;

    /**
     * URL to which requests are POSTed.
     * @var string
     */
    protected $site_verify_url;

    public function __construct(KObjectConfig $config)
    {
        parent::__construct($config);

        $this->captcha         = $config->captcha;
        $this->secret_key      = $config->secret_key;
        $this->site_verify_url = $config->site_verify_url;
    }

    protected function _initialize(KObjectConfig $config)
    {
        parent::_initialize($config);

        $config->append(array(
            'captcha'         => '',
            'secret_key'      => '',
            'site_verify_url' => 'https://www.google.com/recaptcha/api/siteverify'
        ));
    }

    public function isSupported()
    {
        if (empty($this->secret_key) || !$this->captcha) {
            return false;
        }

        return true;
    }

    protected function _beforeSubmit(KControllerContextInterface $context)
    {
        $data = $context->request->data->toArray();
        $url  = $context->request->getReferrer();

        if (empty($data['g-recaptcha-response']))
        {
            $error = 'Please verify that you are not a robot.';
            $context->getResponse()->setRedirect($url, $error, 'error');
            $context->getResponse()->send();

            return false;
        }

        $response = $this->verify($data['g-recaptcha-response']);

        if (!$response['success'])
        {
            $error = $this->getErrorMessage(current($response['error-codes']));
            $context->getResponse()->setRedirect($url, $error, 'error');
            $context->getResponse()->send();

            return false;
        }
    }

    /**
     * Verify the user's response
     *
     * @param string $response
     *
     * @return array The result of
     */
    public function verify($response)
    {
        $handle  = curl_init($this->site_verify_url);
        $params  = array('secret' => $this->secret_key, 'response' => $response);
        $options = array(
            CURLOPT_POST       => true,
            CURLOPT_POSTFIELDS => http_build_query($params, null, '&'),
            CURLOPT_HTTPHEADER => array(
                'Content-Type: application/x-www-form-urlencoded'
            ),
            CURLINFO_HEADER_OUT    => false,
            CURLOPT_HEADER         => false,
            CURLOPT_RETURNTRANSFER => true,
            CURLOPT_SSL_VERIFYPEER => true
        );

        curl_setopt_array($handle, $options);

        $response = curl_exec($handle);

        curl_close($handle);

        return json_decode($response, true);
    }

    public function getErrorMessage($code)
    {
        $error = 'Please verify that you are not a robot.';

        $errors = array(
            'missing-input-secret'   => 'The secret parameter is missing.',
            'invalid-input-secret'   => 'The secret parameter is invalid or malformed.',
            'missing-input-response' => 'The response parameter is missing.',
            'invalid-input-response' => 'The response parameter is invalid or malformed.'
        );

        if (isset($errors[$code])) {
            $error = $errors[$code];
        }

        return $error;
    }
}