<?php
/**
 * @package     Docman.Plugin
 * @subpackage  Leads
 * @copyright   Copyright (C) 2011 - 2016 Timble CVBA. (http://www.timble.net)
 * @license     GNU GPLv3 <http://www.gnu.org/licenses/gpl.html>
 * @link        http://www.joomlatools.com
 */

/**
 * Form Controller Behavior
 *
 * @package Joomlatools\Plugin\DOCman
 */
class PlgDocmanLeadsControllerBehaviorFormable extends KControllerBehaviorAbstract
{
    protected static $_document;
    protected static $_category;

    protected function _initialize(KObjectConfig $config)
    {
        $config->append(array(
            'priority'               => KCommandHandlerInterface::PRIORITY_HIGH,
            'title'                  => '',
            'categories'             => array(),
            'notification_emails'    => '',
            'notification_title'     => '',
            'notification_message'   => '',
            'include_subcategories' => '',
            'params'                 => ''
        ));

        parent::_initialize($config);
    }

    /**
     * Send an email notification and redirect back to download page
     */
    public function _actionSubmit(KControllerContextInterface $context)
    {
        $data = $context->request->data;

        if (isset($data->form))
        {
            $config  = JFactory::getConfig();
            $lead    = $data->form;

            $emails  = KObjectConfig::unbox($this->getConfig()->notification_emails);

            $subject = $this->_prepareText($this->getConfig()->notification_title, $lead);
            $message = $this->_prepareText($this->getConfig()->notification_message, $lead);

            // Email specified recipients
            JFactory::getMailer()->sendMail($config->get('mailfrom'), $config->get('fromname'), $emails, $subject, $message, true);

            // Email Lead
            if($this->getConfig()->params['notify_lead'])
            {
                $lead_email   = $lead['email'];
                $lead_subject = $this->_prepareText($this->getConfig()->params['lead_notification_title'], $lead);
                $lead_message = $this->_prepareText($this->getConfig()->params['lead_notification_message'], $lead);

                JFactory::getMailer()->sendMail($config->get('mailfrom'), $config->get('fromname'), $lead_email, $lead_subject, $lead_message, true);
            }

            // Redirect to thank you message page.
            $url = clone $this->getRequest()->getUrl();
            $url->query['thankyou'] = 1;
            $context->response->setRedirect($url);
        }
    }

    /**
     * Display form on the selected categories.
     *
     * @param KControllerContextInterface $context
     * @return bool
     */
    protected function _beforeRender(KControllerContextInterface $context)
    {
        $config       = $this->getConfig();
        $categories   = KObjectConfig::unbox($config->categories);

        $document     = $this->getModel()->fetch();
        $category_id  = $document->docman_category_id;
        $category_ids = array();

        $menu         = JFactory::getApplication()->getMenu()->getActive();
        $params       = $menu->params;

        // Do not display form for player
        if ($document->isPlayable())
        {
            if (!$params->get('force_download') && $params->get('show_player', 1)) {
                return true;
            }
        }

        if ($config->include_subcategories)
        {
            $category     = $this->getObject('com://site/docman.model.categories')->id($category_id)->fetch();
            $category_ids = $this->_getArrayColumn($category->getAncestors()->toArray(), 'id');
        }

        $category_ids[] = $category_id;

        if (!$categories || array_intersect((array)$category_ids, (array)$categories))
        {
            $url   = clone $this->getRequest()->getUrl();
            $query = $this->getRequest()->query;

            if ($query->download) {
                return true;
            }

            $template = 'file://plugins/docman/leads/view/default';
            $override = 'com://site/docman.leads.default';

            if ($query->thankyou)
            {
                $template = 'file://plugins/docman/leads/view/message';
                $override = 'com://site/docman.leads.message';

                if (!$config->params['notify_lead'])
                {
                    // Download page url.
                    $url = clone $this->getRequest()->getUrl();
                    $url->query['download'] = 1;
                }
                else {
                    $url = '';
                }

                if (!$config->params->get('display_always', false))
                {
                    // Set a cookie
                    $cookie = $this->getObject('lib:http.cookie', array(
                        'name'   => 'docman-leads',
                        'value'  => true,
                        'expire' => strtotime('+1 year')
                    ));
                    $context->response->headers->addCookie($cookie);
                }
             }

            if ($this->getObject('template.locator.factory')->createLocator($override)->locate($override.'.html')) {
                $template = $override;
            }

            $params = $config->params;

            $options = array(
                'title'  => $config->title,
                'url'    => $url,
                'params' => $params
            );

            $gdpr = $params->get('gdpr');

            if ($gdpr->show)
            {
                $translator = $this->getObject('translator');

                if ($translator->getCatalogue()->has('PLG_DOCMAN_LEADS_GDPR_NOTICE')) {
                    $gdpr->notice = $translator->translate('PLG_DOCMAN_LEADS_GDPR_NOTICE'); // Grab from translation file
                } else {
                    $gdpr->notice = $params->get('gdpr_notice', 'We want you to know how our service works and why your details are being submitted. Please confirm that you have carefully read, understood, and agreed to our privacy policy and terms of conditions to proceed with the download.'); // Grab from parameter
                }

                $options['gdpr'] = $gdpr;
            }

            $html = $this->getObject('com:koowa.view.html', array('layout' => $template))->render($options);

            $this->getResponse()->setContent($html);
            return false;
        }

        return true;
    }

    /**
     * Replace shortcodes to actual value
     *
     * @param text string
     * @param lead
     * @return text
     */
    protected function _prepareText($text, $lead)
    {
        $regex = '/\\{[\s]?(document|lead|category):([A-Za-z0-9_\\-]+)[\s]?\\}/';

        preg_match_all($regex, $text, $matches);

        if (count($matches))
        {
            foreach ($matches[0] as $key => $search)
            {
                $type     = $matches[1][$key];
                $property = $matches[2][$key];

                if ($type == 'document') {
                    $item = $this->_getDocument();
                } else if ($type == 'category') {
                    $item = $this->_getCategory($this->_getDocument()->docman_category_id);
                } else {
                    $item = (object) $lead;
                }

                if ($item && isset($item->$property)) {
                    $text = str_replace($search, $item->$property, $text);
                }
            }
        }

        return $text;
    }

    protected function _getDocument()
    {
        if (!self::$_document)
        {
            $document = $this->getModel()->fetch();
            $document->url           = $this->_getUrl($document);
            $document->download_link = $this->_getUrl($document, 'download');

            self::$_document = $document;
        }

        return self::$_document;
    }

    protected function _getCategory($category)
    {
        if (!self::$_category) {
            self::$_category = $this->getObject('com://site/docman.model.categories')->id($category)->fetch();
        }

        return self::$_category;
    }

    protected function _getUrl($document, $type = 'document')
    {
        $Itemid = $this->getObject('request')->query->Itemid;

        if ($type == 'download')
        {
            $url = 'index.php?option=com_docman&view=download&alias=%s&Itemid=%d&download=1';
            $url = sprintf($url, $document->alias, $Itemid);
        } else
        {
            $url = 'index.php?option=com_docman&view=document&alias=%s&category_slug=%s&Itemid=%d';
            $url = sprintf($url, $document->alias, $document->category_slug, $Itemid);
        }

        $url = JApplication::getInstance('site')->getRouter()->build($url);
        $url = rtrim(JURI::root(), '/').$url;

        return $url;
    }

    protected function _getArrayColumn($items, $column)
    {
        $result = array();

        foreach ($items as $item)
        {
            if (isset($item[$column])) {
                $result[] = $item[$column];
            }
        }

        return $result;
    }

}